/*
    NullBot Robot Client 

    (C) 2005-2011 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com

    FILE: main.cpp
    
    Main window
*/

#include <windows.h>
#include <stdio.h>


#include "types.h"
#include "config.h"
#include "main.h"
#include "resource.h"
#include "automode.h"
#include "robot.h"
#include "network.h"
#include "robot.h"
#include "camera.h"
#include "..\version.h"
#include "config.h"
#include "utility.h"
#include "profile.h"

#define MODULE_NAME "[Main] "



/*
    Class Maintenance
*/


/* Main class constructor */
main_class::main_class( void )
{
    log_header();

    memset(text_area, ' ', sizeof(text_area));  // Clear display area


    config_table = NULL;

    load_config(&config_table, "nullbot.conf");

    update_rate = 100;

    display_x = 1024;                           // Inital Display Size
    display_y = 768;
    
    mem_bm = 0;                                 // Clear memory bitman and dc handles
    mem_dc = 0;

    update_display_mode();                      // Make sure display mode is correct

    window_active = 0;                          // Window is not currently active

    current = get_time();                       // Set inital time 

    robot = NULL;

    updates = 0;                                // Number of updates since last second
    updates_per_second = 0;                     // Number of updates per second

    manual_mode = true;

    mouse_is_down = false;
    right_mouse_down = false;
    show_stats = false;

    display_text = true; 
    reset_keys(); // clear set keys
       
    rand_seed();                                // Seed Random number generator

    profile = new profile_class(this);

    parse_config();    

    setup_window();                             // Create window

    memdc_setup(hwnd);                          // Setup buffer

    robot = new robot_class(this);

    SetTimer(hwnd, UPDATE_ID, 1000 / update_rate, NULL);        
    SetTimer(hwnd, SECOND_ID, 1000, NULL);                              
}
/* End of main class constructor */


/* Main class destructor */
main_class::~main_class( void )
{
    log_header();
    
    alert(MODULE_NAME "Main Shutdown");
            
    delete robot;

    delete profile;
    
    DeleteObject(mem_bm);
    DeleteDC(mem_dc);   
    
}
/* End of main class destructor */


/*
    End of Class Maintenance
*/


/* Parse config enteries for this module */
void main_class::parse_config ( void )
{
    config_entry *line = config_get_line(config_table, "main");
    
    log("main parse config\n");
    
    while (line)
    {
        log("Data %s\n", line->data);
    
        unsigned int index = 0;
        char str[CONFIG_LINE_MAX];
    
        get_next_string(line->data, index, str);

        if (!strcmp(str, "display"))
        {
            get_next_string(line->data, index, str);
            int x = atoi(str);
            
            get_next_string(line->data, index, str);
            int y = atoi(str);
            
            set_display(x, y);
            log(MODULE_NAME "Set display to [%d,%d]\n", x, y);

        }    

        if (!strcmp(str, "update_rate"))
        {
            get_next_string(line->data, index, str);
            update_rate = atoi(str);
            log(MODULE_NAME "Set update_rate to %d\n", update_rate);
        }    
        
        line = line->next_line;
    }
}
/* End of parse_config */

/*
    Timers and Updates
*/


/* Deal with the system timer */
void main_class::timer_message(HWND hwnd, WPARAM wparam)
{
    log_header();

    if (wparam == UPDATE_ID)
    {
        current = get_time();

    
        static unsigned int count = 0;

        if (count++ > 10)
        {
            robot->update();
            updates++;
        }        


        // Manually redraw at UPDATE_RATE so cameras
        // This keeps the cameras from forcing too many paints
        InvalidateRect (robot->main->get_hwnd(), NULL, FALSE);         
        
    } 

    if (wparam == SECOND_ID)
    {
        robot->check(); 
              
        updates_per_second = updates;
        updates = 0;    
        
    }            
}
/* End of timer_message */


/* Check window activity */
void main_class::check_active(int active)
{
    log_header();

    window_active = active;
    
    if (window_active == 0)
    {
        reset_keys();    
    
        if (robot)
           robot->get_comm()->motion_stop();
    }
     
}
/* End of check active */

/*
    End of Timers and Updates
*/



/*
    Graphics
*/


/* Update internal display offsets */
void main_class::update_display_mode( void )
{
    log_header();

    if (display_text)
        display_y_act = display_y - font_y_size*text_y_size - 4;          
    else
        display_y_act = display_y;
}
/* End of update_display_mode */


/* Adds a line to the text buffer */
void main_class::add_alert(char *line)
{
    static unsigned int text_y = 0;                            // current y_pos in text buffer

    // scrool the display up
    if (text_y == text_y_size)
    {
        for (int i = 1; i < text_y_size; i++)
            memcpy(text_area[i-1], text_area[i], text_x_size);
    }
    else
        text_y ++;

    // erase the line
    memset(text_area[text_y-1], ' ', text_x_size);

    // copy the string
    unsigned int line_len = strlen(line);

    if (line_len > text_x_size)
        line_len = text_x_size;

    memcpy(text_area[text_y-1], line, line_len);

    // Update memdc
    Rectangle(mem_dc, 0, display_y_act, display_x, display_y);    
    
    SelectObject(mem_dc, GetStockObject(SYSTEM_FIXED_FONT));

    // draw text lines
    for (unsigned int r = 0; r < text_y_size; r++)
        TextOut(mem_dc, 1, display_y_act + r * font_y_size + 1, text_area[r], text_x_size);
}
/* End of add line */


/* Write formatted data to the debugging console */
void main_class::alert(char *fmt, ... )
{
    const unsigned int max = 1024;

    va_list args;
    char text[max];

    char out_str[max] = {0};
  
    va_start(args, fmt);
    vsnprintf(text, max, fmt, args);
    
    add_alert(text);
    va_end(args);    
    
    log(MODULE_NAME "%s\n", text);
}
/* End of write_console */


/* Deal with paint message */
void main_class::window_paint(HWND hwnd1)
{
    log_header();

    PAINTSTRUCT ps;
    HDC hdc = BeginPaint(hwnd, &ps);

    char buff[400];

  //  if (show_stats)
    {
    
        if (robot)
        {
            sprintf(buff, "Wifi Signal: %4.1f  %4.1f ",  
                          robot->get_wifi_signal(), robot->get_average_fps() );
            TextOut(mem_dc, 1, 1, buff, strlen(buff)); 
        }
    }          
            
    // Copy memory device context to display 
    BitBlt(hdc, 0, 0, display_x, display_y, mem_dc, 0, 0, SRCCOPY);         
           
    EndPaint(hwnd, &ps);     
}
/* End of window_paint */


/* Setup the off screen drawing system */
void main_class::memdc_setup( HWND hwnd )
{
    log_header();

    RECT r;
    GetClientRect(hwnd, &r);

    // Update stored client area size
    display_x = r.right - r.left;
    display_y = r.bottom - r.top;
    
    update_display_mode();

    log(MODULE_NAME "display [%d,%d]\n", display_x, display_y);


    HDC hdc = GetDC(hwnd);

    // Create memory device context and bitmap            
    mem_dc = CreateCompatibleDC(hdc);
    mem_bm = CreateCompatibleBitmap(hdc, r.right, r.bottom);
            
    // Select the bitmap into the memort dc
    SelectObject(mem_dc, mem_bm);
                        
    Rectangle(mem_dc, 0, 0, display_x, display_y);                
                    
    ReleaseDC(hwnd, hdc);     
}
/* End of memdc_setup */


/* Set new window size */
void main_class::set_display( int dx, int dy)
{
    display_x = dx;
    display_y = dy;
}
/* End of main_class::set_display */


/* Called when we resize */
void main_class::resize_window(unsigned int x_size, unsigned int y_size)
{
    log_header();

    // These are already setup, delete them
    if (mem_bm) DeleteObject(mem_bm);
    if (mem_dc) DeleteDC(mem_dc);

    // Setup again with the new size
    memdc_setup(hwnd);
}
/* End of resize */


/*
    End of Graphics
*/


/*
    Windows Related
*/


/* Deal with main menu commands */
void main_class::main_menu(HWND hwnd, unsigned int id)
{
    log_header();

    switch(id)
    {
        case IDM_SERVER: robot->get_comm()->get_servername(); break;

        case IDM_CONNECT: robot->get_comm()->do_connect(); break;

        case IDM_DISCONNECT: robot->get_comm()->disconnect(); break;

        case IDM_STOP: robot->get_comm()->set_noconnect(1); break;

        case IDM_EXIT: PostQuitMessage(0); break;

        case IDM_SHUTDOWN: robot->get_comm()->remote_shutdown(); break;
    }    
}
/* End of main menu */


/* Wrapper for the camera window procedure */
LRESULT CALLBACK main_class::proc_wrapper(HWND hwnd, UINT message, WPARAM w, LPARAM l)
{
    // Get object pointer from window data and call actual wndproc
        
    // Set the class pointer on creation
    if (message == WM_NCCREATE)
    {     
        if(l)
        {
            LONG handle = (LONG)((CREATESTRUCT*)l)->lpCreateParams;
        
            if(handle)
                SetWindowLong(hwnd, GWL_USERDATA, handle);
        }
    }
    
    main_class *win = (main_class*)::GetWindowLong(hwnd, GWL_USERDATA);
            
    // Call the member function     
    win -> proc(hwnd, message, w, l);
}
/* End of cam_proc_wrapper */


/* Window function */
LRESULT CALLBACK main_class::proc(HWND hwnd, UINT message, WPARAM wparam, LPARAM lparam)
{
    // Decode network even for camera
    if (message >= WM_CAMEVENT && message <= WM_CAMEVENT + 16)
    { 
        int index = (int)message - WM_CAMEVENT;
                
        if (index >= 0 && index < robot->get_num_cameras())
            robot->camera_message(index, wparam, lparam);

       return (0);
    }

    switch(message)
    {
        case WM_NETEVENT:       robot->get_comm()->control_socket_message(wparam, lparam); break;
        case WM_TIMER:          timer_message(hwnd, wparam); break;
        case WM_PAINT:          window_paint(hwnd); break;
        case WM_DESTROY:        PostQuitMessage(0); break;
        case WM_SIZE:           resize_window(lparam & 0xffff, lparam >> 16);  break;          // Process size message                
        case WM_KEYDOWN:        process_key_down(wparam & 0xff);  break; // Process keydown
        case WM_KEYUP:          process_key_up(wparam & 0xff);  break; // Process keyup
        case WM_COMMAND:        main_menu(hwnd, LOWORD(wparam));   break;
        case WM_ACTIVATEAPP:    check_active(wparam); break;
        case WM_KILLFOCUS:      check_active(0);   break;
                
        default:
            return(DefWindowProc(hwnd, message, wparam, lparam));    
    }
    return(0);
}
/* End of main_class::proc */


/* Create the main window */
void main_class::setup_window( void )
{
    log_header();

    WNDCLASS wc;
    
    HINSTANCE  hinst = 0;

  
    wc.hInstance = hinst;
    wc.lpszClassName = "NullBotWin";
    wc.lpfnWndProc = proc_wrapper;
    wc.style = 0;
    
    wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.lpszMenuName = "MAINMENU";
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
        
    wc.hbrBackground =  (HBRUSH)GetStockObject(WHITE_BRUSH);
    
    char name[256];
    sprintf(name, "NullBot Remote Sensor Platform. Version %s %s ", VERSION, COPYRIGHT);
    
    if (!RegisterClass(&wc)) return ;

    hwnd = CreateWindowEx (0, wc.lpszClassName,
                                name,  
                                WS_OVERLAPPEDWINDOW,  //  &~WS_THICKFRAME ,
                                20, 20, // x, y
                                display_x,display_y, // height, width
                                HWND_DESKTOP, NULL,
                                NULL, this);
                            
    ShowWindow(hwnd, 1);
    UpdateWindow(hwnd);
}
/* End of main_class::setup_window */


/* Winmain */
int WINAPI WinMain(HINSTANCE hinstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
    log_header();

    WSADATA wsda;
    WSAStartup(MAKEWORD(1, 1),&wsda);

    main_class *main = new main_class;        // Create main window
    
    main->setup_window();                     // Setup main window
   
    // Message loop
    MSG messages;

    while (GetMessage (&messages, NULL, 0, 0))
    {
        TranslateMessage(&messages);
        DispatchMessage(&messages);
    }
    
    delete main;
    
    WSACleanup();

    return(messages.wParam);
}
/* End of WinMain */

/*
    End of Windows Related
*/





