/*
    NullBot Robot Client 

    (C) 2005-2011 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com

    FILE: main.h
*/


#ifndef MAIN_H
#define MAIN_H


#define UPDATE_ID 101
#define SECOND_ID 102

#define max_image_x 640     // size of image buffer
#define max_image_y 480

#define text_x_size 100
#define text_y_size 6

#define font_x_size 8
#define font_y_size 16


#define text_area_height  (text_y_size*font_y_size) 


#define ID_EDIT_CONSOLE 0



class profile_class;
class robot_class;

class main_class 
{
    public:
    
    main_class( void );
    ~main_class( void );
    
    void parse_config ( void );
    
    void setup_window( void );
    
   // void main_startup( void );
   // void main_shutdown( void );
    void key_down(HWND hwnd, WPARAM w);
    void main_menu(HWND hwnd, unsigned int id) ;
    void key_up(WPARAM w)  ;
    void check_keys( void )  ;
    void mouse_down(LPARAM l);
    void mouse_up(LPARAM l);
    void mouse_move(LPARAM l) ;
    void timer_message(HWND hwnd, WPARAM wparam)  ;
    void window_paint(HWND hwnd)  ;
    void memdc_setup( HWND hwnd )  ;
        
    void check_active(int active);
       
        
    
    void update_display_mode( void );        
        
    void reset_keys( void );

    void update_movement( void  );
    void process_key_down(unsigned char key);
    void process_key_up( unsigned char key );
      
        
    void resize_window(unsigned int x_size, unsigned int y_size);
        
        
    static LRESULT CALLBACK proc_wrapper(HWND hwnd, UINT message, WPARAM w, LPARAM l);
    LRESULT CALLBACK proc(HWND hwnd, UINT message, WPARAM wparam, LPARAM lparam)      ;   
           
    int get_display_x(void) { return display_x; };
    int get_display_y(void) { return display_y; };
    int get_display_y_act(void) { return display_y_act; };
   


    void add_alert(char *line);
    void alert(char *fmt, ... );

    void set_display( int dx, int dy);
    
    HDC get_memdc( void ) { return mem_dc; };
 
    HWND get_hwnd( void ) { return hwnd; };
 
    double get_current(void) { return current; } ;



    profile_class * get_profile( void ) { return profile; } ;


    config_entry * get_config( void ) { return config_table; };


  private:
    // Mouse           
    unsigned int mouse_x_start;
    unsigned int mouse_y_start;
    bool mouse_is_down ;
    bool right_mouse_down ;

    // Keyboard
    bool keys[256];     // Key array, true if pressed

    // Stats
    int updates ;                // Number of updates since last second
    int updates_per_second ;     // Number of updates per second
    bool show_stats;

    double current; // current time

    bool manual_mode ;

    // Double Buffer
    HDC mem_dc;                   // Device context for offscreen drawing
    HBITMAP mem_bm;               // Bitmap for memdc


    // Debug display
    bool display_text;              // true if you want to see debugging iinformation
    char text_area[text_y_size][text_x_size];  // the data for textr display

    // Robot Control
    double gui_speed;               // Motion command scale



    // Window variables
    unsigned int window_active;

    int display_x;
    int display_y;
    int display_y_act;


    int update_rate;

    HWND hwnd;


    config_entry *config_table;


    profile_class *profile;

  public:
    robot_class *robot;         // All robots
  
};

#endif












