/*
    NullBot Robot Client 

    (C) 2005-2011 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
    FILE: robot.h
*/


#ifndef ROBOT_H
#define ROBOT_H

#define NAME_MAX    256
#define MAX_CAMERAS  32
#include "main.h"



class robot_class
{
  public:
  
    robot_class( main_class *m );
    ~robot_class();

    void parse_config ( void );


    void update( void );
    void check( void );

    char * get_host_name (void) { return host_name; } ;
    unsigned int get_base_port( void ) { return base_port; };
    void set_host_name(char *name);
    void set_name(char *name);


    // Telemtry
    void process_telemetry(unsigned char *bf, unsigned int len);
    void telemetry_string(char *st);

    
    // Control 
    int get_control_data(unsigned char *buff);
    void stop_motion( void);
    void motion(double s, double d);
    void move_pan( int v );
    void set_pan( int v );
    void set_tilt( int v );
    int get_pan( void ) { return pan; };
    int get_tilt( void ) { return tilt; };


    // Automode
    void robot_automode( void );
    void toggle_automode( void );
    bool get_automode( void ) { return auto_mode; };
    void set_auto_mode ( int m) { auto_mode = m; };
    






    void robot_class::update_motion(void);



    main_class * get_main(void) { return main; };



    network_class * get_comm( void) {  return comm; };


    int get_num_cameras(void) { return num_cameras;};

    void camera_message(int index, WPARAM w, LPARAM l);

    double get_wifi_signal( void ) { return wifi_signal; };


    double get_average_fps( void ) ;


    bool get_targetting( void ) { return targetting; };
    void set_targetting( bool t ) { targetting = t; };


  private:
    double ddir;                 // Facing direction
    double speed;
    
    double target_speed;
    
    double accel_1;
    double accel_2;
    
    
    double wifi_signal;

    unsigned int base_port;

    char name[NAME_MAX];

    // motor controller values
    unsigned char pan , tilt;

    unsigned char servo_0;
    unsigned char servo_1;
    unsigned char servo_2;
    unsigned char servo_3;

    // telemtry data
    float z_angle;
    float x_vel;
    float y_vel;
    float ir_dist;


    bool auto_mode;             // In auto mode
   
    double last_update;

    char host_name[NAME_MAX];
    network_class *comm;            // Child


    int num_cameras;
    camera_class *cameras[MAX_CAMERAS];


    bool targetting;
    
    
  public:

    main_class *main;                  // Parent 
    
    
    
};


#endif








