/*
    NullBot Robot Client 

    (C) 2005-2011 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
    
    Simple target tracking system
*/


#include <windows.h>
#include <stdio.h>
#include <math.h>

#include "types.h"
#include "config.h"
#include "main.h"



#include "target.h"
#include "automode.h"

#include "robot.h"
#include "camera.h"
#include "network.h"
#include "camera.h"



#define MODULE_NAME "[Target] "



/* Compute change between frames */
unsigned char camera_class::get_change(unsigned int x, unsigned int y)
{
    unsigned int index = x + y * image_x;

    unsigned int change = (int)(fabs(camera_buffer[index].r - last_frame[index].r) + 
                                fabs(camera_buffer[index].g - last_frame[index].g) +
                                fabs(camera_buffer[index].b - last_frame[index].b)) / 3;

    if (change < 50) // 50 works
        change = 0;

    return((unsigned char )fabs(change));
}
/* End of get_change */
 


/* Update pan and tilt based on camera data */
void camera_class::process_frame( void )
{
    unsigned int sum_x = 0, sum_y = 0, num = 0, index = 0;

    // compute centroid of difference frame
    // with weighted averages
    for (unsigned int y = 0; y < image_y; y++)
        for (unsigned int x = 0; x < image_x; x++)
        {
            unsigned int  v = get_change(x, y);

            if (d_frame) // if we need to display the difference frame
            {
                unsigned int tmp = v;
                if (tmp > 255) tmp = 255;

                processed_frame[index].g = (unsigned char)tmp;
                processed_frame[index].r = 0;        
                processed_frame[index].b = 0;

                index++;
            }

            sum_x += x*v;
            sum_y += y*v;
            num+=v;       
        }

    // waiting for camera to stabilize
  //  if (update++ < 10) 
    {
        // save the frame
        memcpy(last_frame, camera_buffer, image_x*image_y * sizeof(color_type));  
        
        if (d_frame)
            memcpy(camera_buffer, processed_frame, sizeof(color_type)*image_x*image_y);

      //  return;
    }

    // center of image
    unsigned int center_x, center_y;
    bool found = false;

    // make sure we have enough change to even bother moving
   if (num > 1500)
//    if (num > 10000)
    {
        center_x = sum_x / num;
        center_y = sum_y / num;
        found = true;
    } 
    else
    {
        center_x = image_x / 2;
        center_y = image_y / 2;
    }

    // display for debugging
//    char b[300];
//    sprintf(b, "num: %d ", num);
//    alert(b);
    
    // done with image, save to old image
    memcpy(last_frame, camera_buffer, image_x*image_y * sizeof(color_type));   

    if (found)
    {
        unsigned int index = center_x;

        // draw lines
        for (unsigned int y = 0; y < image_y; y++, index += image_x)
            camera_buffer[index].g = 255;        

        index = center_y * image_x;

        for (unsigned int x = 0; x < image_x; x++)
            camera_buffer[index++].g = 255;        

        // draw targetting box
        draw_box(center_x-5,center_y-5, center_x+5, center_y+5, camera_buffer);
        
        //robot->comm->fire_on();
    }
   // else
        //robot->comm->fire_off();

    int new_pan = robot->get_pan();
    int diff = center_x - image_x/2;

    // calculate new servo offset
    new_pan -= (int)((diff / (double)(image_x / 2)) * 30);

    // range check        
    if (new_pan < 0) new_pan = 0;
    if (new_pan > 255) new_pan = 255;

    // tilt has not been tested, no tilt servo installed at the time
    int new_tilt = robot->get_tilt();
    diff = center_y - image_y/2;

    // calculate new servo offset
    new_tilt -= (int)((diff / (double)(image_y / 2)) * 30);
        
    // range check
    if (new_tilt < 0) new_tilt = 0;
    if (new_tilt> 255) new_tilt = 255;

    // did any value change?
    if (new_pan != robot->get_pan() || new_tilt != robot->get_tilt())
    {
        //robot->set_pan ( new_pan );
       // robot->set_tilt ( new_tilt );

       // robot->comm->send_controller_values();
        //update = 0;                 // we moved the camera, is has to stablize again
    }
        
    if (d_frame)
        memcpy(camera_buffer, processed_frame, sizeof(color_type)*image_x*image_y);
}
/* End of process frame */


