/*
    Robot server - 2005 Jason Hunt
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
    
    File: Cameratest.c
*/


#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <linux/fb.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <errno.h>
#include <sys/time.h>
#include <time.h>s

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>

#include "../Node/camera_drv.h"
#include "decompress.h"


unsigned char done = 0;


int fbfd = 0;
struct fb_var_screeninfo vinfo;
struct fb_fix_screeninfo finfo;
long int screensize = 0;
unsigned char *fbp = 0;
int x = 0, y = 0;
long int location = 0;



static unsigned short            red[256],  green[256],  blue[256];
static struct fb_cmap            cmap  = { 0, 256, red,  green,  blue, NULL };



/* Set palette for 8-bit mode */
static void set_palette(int fb)
{
   int r, g, b, i;
   
    printf("vinfo.bits_per_pixel %d\n", vinfo.bits_per_pixel );
   
    if (vinfo.bits_per_pixel != 8)
        return;

    i = 0;
   
    if (ioctl(fb, FBIOGETCMAP, &cmap) == -1)
        fprintf(stderr,"ioctl FBIOGETCMAP");
   
    /* generate the palette */
    for (r = 0; r < 8; r++)
    {
        for (g = 0; g < 8; g++)
        {
            for (b = 0; b < 4; b++)
            {
                int val;
                  
                val = (r << 5) | (r << 2) | (r >> 1);

                red[i] = (val << 8) | val;

                val = (g << 5) | (g << 2) | (g >> 1);

                green[i] = (val << 8) | val;

                val = (b << 6) | (b << 4) | (b << 2) | (b);

                blue[i] = (val << 8) | val;

                i++;
            }
        }
    }
   
   /* set colormap */
   if (ioctl(fb, FBIOPUTCMAP, &cmap) == -1)
      fprintf(stderr,"ioctl FBIOPUTCMAP");
}



/* Hangle signals */
static void sighandler(int sig)
{
    done = 1;
}
/* End of sighandler */



int vga_start (void)
{
    fbfd = open("/dev/fb0", O_RDWR);

    if (fbfd == -1) 
    {
        fprintf(stderr,"Error: cannot open framebuffer device.\n");
        return 1;
    }    
    
    
	fcntl(fbfd , F_SETFD, fcntl(fbfd , F_GETFD) | FD_CLOEXEC);    
    
    if (ioctl(fbfd, FBIOGET_FSCREENINFO, &finfo)) 
    {
        fprintf(stderr,"Error reading fixed information. Buffer: %d  Error (%d)\n", fbfd, errno);
        return 1;
    }
    
    	if (finfo.type != FB_TYPE_PACKED_PIXELS) {
		fprintf(stderr, "unsupported frame buffer device!\n");
		return 0;
	}
	

    if (ioctl(fbfd, FBIOGET_VSCREENINFO, &vinfo)) 
    {
        fprintf(stderr,"Error reading variable information.\n");
        return 1;
    }

    screensize = vinfo.xres * vinfo.yres * vinfo.bits_per_pixel / 8;

    fbp = (unsigned char *)mmap(0, screensize, PROT_READ | PROT_WRITE, MAP_SHARED, fbfd, 0);

    if ((int)fbp == -1) 
    {
        fprintf(stderr,"Error: failed to map framebuffer device to memory.\n");
        return 1;
    }


    set_palette( fbfd );


    fprintf(stderr,"vinfo.xoffset: %d\n", vinfo.xoffset);
    fprintf(stderr,"vinfo.yoffset: %d\n", vinfo.yoffset);
    fprintf(stderr,"vinfo.bits_per_pixel: %d\n", vinfo.bits_per_pixel);
    fprintf(stderr,"finfo.line_length: %d\n", finfo.line_length);
    
    fprintf(stderr,"vinfo.xres: %d\n", vinfo.xres);
    fprintf(stderr,"vinfo.yres: %d\n", vinfo.yres);
     





 
    return 0;

}


int vga_stop (void)
{
    munmap(fbp, screensize);
    close(fbfd);

    return 0;
}


int main( void )
{
    unsigned char *buff;

    unsigned int image_x = 0;
    unsigned int image_y = 0;
    unsigned int max_image = 0;

    unsigned int screen_x = 0;
    unsigned int screen_y = 0;
   
   
    signal(SIGABRT, &sighandler);
	signal(SIGTERM, &sighandler);
	signal(SIGINT,  &sighandler);
	
    cameradrv_start ("/dev/video0", &image_x, &image_y, &max_image);
               
    fprintf(stderr,"Image_x: %d\n", image_x);
    fprintf(stderr,"Image_y: %d\n", image_y);
    fprintf(stderr,"Max image: %d\n", max_image);
  

    if (vga_start())
    {
        fprintf(stderr,"Unable to start video\n");
        return 1;
    }
    
    
    screen_x = vinfo.xres;
    screen_y = vinfo.yres;
    

    unsigned char  *raw_image = (unsigned char  *)  malloc(image_x * image_y * 18);
  
   
    buff = (unsigned char *) malloc(max_image * 18);
   
    unsigned int len;
      


    int frames = 0;

    
    struct timeval start, current;


    gettimeofday(&start, NULL);
    
    
    double ave_fps = 0;

    while (!done)
    {
        if (frames > 10)
        {
            gettimeofday(&current, NULL);

            double  diff = (current.tv_sec - start.tv_sec) + (current.tv_usec - start.tv_usec)/1000000.0;
       
            if (diff > 0)
            {
                double a = 0.95;
            
                ave_fps = a * (frames / diff) + (1-a) * ave_fps;

                printf("FPS: %f\n", ave_fps);    
            }

            start = current;
            frames = 0;
        }
    
    
        cameradrv_frame (buff, max_image, &len);
        frames++;
    

        // debug save to file
        //        printf("Len: %d\n", len);
        /*        FILE *f = fopen("image.jpg", "wb");
            fwrite(buff, 1, len, f);
            fclose(f);*/

        decompress_image((char *) buff, len, raw_image, image_x, image_y);
                      
        int x, y;
	


        long int location = 0;
	
location = 0;

        for (y = 0; y < image_y ; y++)
        {
            for (x = 0; x <image_x ; x++) 
            {
                unsigned int addr_x = x;// * image_x / screen_x;
                unsigned int addr_y = y;// * image_y / screen_y;

               // location = (x+vinfo.xoffset) * (vinfo.bits_per_pixel/8) +
                //           (y+vinfo.yoffset) * finfo.line_length;


                unsigned int p = addr_x*3 + addr_y * image_x*3 + 0;


                unsigned char r = raw_image [p + 0];
                unsigned char g = raw_image [p + 1];
                unsigned char b = raw_image [p + 2];

                if (vinfo.bits_per_pixel == 32) 
                {
                    *(fbp + location) = r ;        
                    *(fbp + location + 1) = g ;    
                    *(fbp + location + 2) = b;   
                    *(fbp + location + 3) = 0;
                    location += 4;

                } else  
                if (vinfo.bits_per_pixel == 24) 
                 { //assume 16bpp
                    *(fbp + location) = r ;        
                    *(fbp + location + 1) = g ;  
                    *(fbp + location + 2) = b;  
                    location += 3;
                                
                } else
                if (vinfo.bits_per_pixel == 16) 
                { 
                    unsigned short int t = ((r>>3)<<11) | ((g>>2) << 5) | (b>>3);
                
                    *((unsigned short int*)(fbp + location)) = t;
                    location += 2;

                } else

                if (vinfo.bits_per_pixel == 8)   // Convert to 16 color greyscale for 8 bit
                 { 
                    unsigned int v = (r + g + b) / 3 / 16  + 16;
                    
                    *(fbp + location) =    ((r /32) << 5) | ((g /32) << 2) | (b >> 6);     
                    location++;
                } 
                
                
        
            }
        }
    }
    

    cameradrv_stop ();
    
    free(buff);
    
    free(raw_image);
    
    vga_stop();
      
    printf("Program exit  \n");

    return(0);
}




