/*
    NullBot Robot Platform - Server - 2005-2011 Jason Hunt
    
    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
    
    File: process.c
*/

// Project headers
#include "process.h"
#include "camera_drv.h"

// System Headers
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/*******************************************************************************#
#  huffman tables for Jpeg encoder/decoder                                      #
#           Paulo Assis <pj.assis@gmail.com>                                    #
********************************************************************************/

#define JPG_HUFFMAN_TABLE_LENGTH 0x01A0

static const unsigned char JPEGHuffmanTable[JPG_HUFFMAN_TABLE_LENGTH] = 
{
	// luminance dc - length bits	
	0x00, 
	0x00, 0x01, 0x05, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x00,
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
	// luminance dc - code
	0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09,
	0x0A, 0x0B, 
	// chrominance dc - length bits	
	0x01, 
	0x00, 0x03, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 
	0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
	// chrominance dc - code
	0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09,
	0x0A, 0x0B, 
	// luminance ac - number of codes with # bits (ordered by code length 1-16)
	0x10,
	0x00, 0x02, 0x01, 0x03, 0x03, 0x02, 0x04, 0x03, 0x05, 0x05, 
	0x04, 0x04, 0x00, 0x00, 0x01, 0x7D,
	// luminance ac - run size (ordered by code length)	
	0x01, 0x02, 0x03, 0x00, 0x04, 0x11, 0x05, 0x12, 0x21, 0x31,
	0x41, 0x06, 0x13, 0x51, 0x61, 0x07, 0x22, 0x71, 0x14, 0x32,
	0x81, 0x91, 0xA1, 0x08, 0x23, 0x42, 0xB1, 0xC1, 0x15, 0x52,
	0xD1, 0xF0, 0x24, 0x33, 0x62, 0x72, 0x82, 0x09, 0x0A, 0x16,
	0x17, 0x18, 0x19, 0x1A, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A,
	0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x43, 0x44, 0x45,
	0x46, 0x47, 0x48, 0x49, 0x4A, 0x53, 0x54, 0x55, 0x56, 0x57,
	0x58, 0x59, 0x5A, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69,
	0x6A, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x83,
	0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8A, 0x92, 0x93, 0x94,
	0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0xA2, 0xA3, 0xA4, 0xA5,
	0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6,
	0xB7, 0xB8, 0xB9, 0xBA, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7,
	0xC8, 0xC9, 0xCA, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8,
	0xD9, 0xDA, 0xE1, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8,
	0xE9, 0xEA, 0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8,
	0xF9, 0xFA, 
	// chrominance ac -number of codes with # bits (ordered by code length 1-16)
	0x11, 
	0x00, 0x02, 0x01, 0x02, 0x04, 0x04, 0x03, 0x04, 0x07, 0x05,
	0x04, 0x04, 0x00, 0x01, 0x02, 0x77,
	// chrominance ac - run size (ordered by code length)
	0x00, 0x01, 0x02, 0x03, 0x11, 0x04, 0x05, 0x21, 0x31, 0x06,
	0x12, 0x41, 0x51, 0x07, 0x61, 0x71, 0x13, 0x22, 0x32, 0x81,
	0x08, 0x14, 0x42, 0x91, 0xA1, 0xB1, 0xC1, 0x09, 0x23, 0x33,
	0x52, 0xF0, 0x15, 0x62, 0x72, 0xD1, 0x0A, 0x16, 0x24, 0x34,
	0xE1, 0x25, 0xF1, 0x17, 0x18, 0x19, 0x1A, 0x26, 0x27, 0x28,
	0x29, 0x2A, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x43, 0x44,
	0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x53, 0x54, 0x55, 0x56,
	0x57, 0x58, 0x59, 0x5A, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68,
	0x69, 0x6A, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A,
	0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8A, 0x92,
	0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0xA2, 0xA3,
	0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xB2, 0xB3, 0xB4,
	0xB5, 0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xC2, 0xC3, 0xC4, 0xC5,
	0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6,
	0xD7, 0xD8, 0xD9, 0xDA, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7,
	0xE8, 0xE9, 0xEA, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8,
	0xF9, 0xFA
};





/* Insert a default huffman table. */
// Code adapted from guvcview
void process_image(camera_type *camera, unsigned char *image_in, 
                                        unsigned char *image_out, 
                                        unsigned int in_len, 
                                        unsigned int *out_len)
{

//	int ret=0;
	int jpgsize=0;
//	unsigned char  *jpgtmp=NULL;
	unsigned char  *Pimg=NULL;
	unsigned char  *Pjpg=NULL;
	unsigned char  *tp=NULL;	
	JPGFILEHEADER JpgFileh;
	int jpghsize=sizeof(JpgFileh);
	//FILE *fp;

	
	jpgsize=jpghsize+in_len+sizeof(JPEGHuffmanTable)+4;/*header+huffman+marker+buffsize*/

	Pimg=image_in;
	Pjpg=image_out;
	/*Fill JFIF header*/
	JpgFileh.SOI[0]=0xff;
	JpgFileh.SOI[1]=0xd8;
	JpgFileh.APP0[0]=0xff;
	JpgFileh.APP0[1]=0xe0;
	JpgFileh.length[0]=0x00;
	JpgFileh.length[1]=0x10;
	JpgFileh.JFIF[0]=0x4a;//JFIF0
	JpgFileh.JFIF[1]=0x46;
	JpgFileh.JFIF[2]=0x49;
	JpgFileh.JFIF[3]=0x46;
	JpgFileh.JFIF[4]=0x00;
	JpgFileh.VERS[0]=0x01;//version 1.2
	JpgFileh.VERS[1]=0x02;
	JpgFileh.density=0x00;
	JpgFileh.xdensity[0]=0x00;
	JpgFileh.xdensity[1]=0x78;
	JpgFileh.ydensity[0]=0x00;
	JpgFileh.ydensity[1]=0x78;
	JpgFileh.WTN=0;
	JpgFileh.HTN=0;

	/*adds header (JFIF)*/
	memmove(Pjpg,&JpgFileh,jpghsize);
	/*moves to the end of the header struct (JFIF)*/
	Pjpg+=jpghsize;
	int headSize = image_in[4]*256+image_in[5] + 4;/*length + SOI+APP0*/
	/*moves to the end of header (MJPG)*/
	Pimg+=headSize;
	/*adds Quantization tables and everything else until   * 
	* start of frame marker (FFC0) 	 */
	tp=Pimg;
	int qtsize=0;
	while(!((tp[qtsize]== 0xff) && (tp[qtsize+1]== 0xc0))) 
	{
		qtsize++;
	}
	memmove(Pjpg,Pimg,qtsize);
	/*moves to the begining of frame marker*/
	Pjpg+=qtsize; 
	Pimg+=qtsize;
	/*insert huffman table with marker (FFC4) and length(x01a2)*/
	BYTE HUFMARK[4];
	HUFMARK[0]=0xff;
	HUFMARK[1]=0xc4;
	HUFMARK[2]=0x01;
	HUFMARK[3]=0xa2;
    memmove(Pjpg,&HUFMARK,4);
	Pjpg+=4;
	memmove(Pjpg,&JPEGHuffmanTable,JPG_HUFFMAN_TABLE_LENGTH);/*0x01a0*/
	/*moves to the end of huffman tables (JFIF)*/
	Pjpg+=JPG_HUFFMAN_TABLE_LENGTH;
	/*copys frame data(JFIF)*/
	memmove(Pjpg,Pimg,(in_len-(Pimg-image_in)));
	Pjpg+=in_len-(Pimg-image_in);
	
	int totSize = Pjpg - image_out;
	

	Pimg=NULL;
	Pjpg=NULL;
	tp=NULL;

    *out_len = totSize;
}






