/*
    NullBot Robot Platform - Server - 2005-2011 Jason Hunt
    
    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
    
    File: robot.c
*/

#include <stdio.h>
#include <memory.h>
#include <termios.h> 
#include <fcntl.h> 
#include <unistd.h>
#include <pthread.h>

#include "../netcomm.h"
#include "utility.h"
#include "serial.h"
#include "failsafe.h"
#include "server.h"
#include "robot.h"
#include "network.h"

#define MODULE_NAME "[Robot] "


robot_type robot;


void robot_get_wifi_strength ( void )
{
    FILE *f = popen(" awk -F'[ :]*' '/:/ {gsub(/\\. /,\" \",$0); print $4; }' /proc/net/wireless","r");

    if(f == NULL)
    {
        alert("Unable to open wifi file\n");
        return;
    }

    unsigned int quality;

    fscanf(f, "%d", &quality);

 //   alert("Link: %3.0f%%\n",  quality/LINK_QUALITY_MAX*100 );

    robot.wifi_link = (unsigned int)(quality * 255 /LINK_QUALITY_MAX);

    fclose(f);   
}



/* Return true if the client has been updated */
unsigned int robot_check_client( void )
{
    unsigned int tmp = robot.client_updates;
    
    robot.client_updates  = 0;
    
    return tmp;
}
/* End of Robot_check_client */


/* Return true if the client has been updated */
unsigned int robot_check_serial( void )
{
    unsigned int tmp = robot.serial_updates;
    
    robot.serial_updates  = 0;
    
    return tmp;
}



/* End of robot_check_serial */
void robot_send_telemetry( void )
{
//    alert(MODULE_NAME "send_telemetry\n");
  

    unsigned char data_size = 0;
    unsigned char buffer[255];


    memcpy(buffer, robot.raw_telemetry, COMM_IN_TEL_LEN);


    data_size += COMM_IN_TEL_LEN;

    robot_get_wifi_strength();

    
    buffer[ data_size++] = robot.wifi_link;
    buffer[ data_size++] = 1;
    buffer[ data_size++] = 2;
    buffer[ data_size++] = 3;
        
         

    network_write_packet(COMM_NC_TEL, buffer, data_size);
}






 
/* Set telemeter data from buffer */
void robot_parse_telemetry( unsigned char * buffer, unsigned int len )   // telemetry into buffer
{

/*    double rc = 0.05;
    double val;*/
        

    if (len != COMM_IN_TEL_LEN) 
    {
        alert(MODULE_NAME "Invalid telemetry length\n");
        return;
    }

//     show_packet("Tele Buffer: ", buffer, COMM_IN_TEL_LEN);



    // Save for transfer to client
    memcpy(robot.raw_telemetry, buffer, COMM_IN_TEL_LEN);
                 
        
    // Extract data for local use
    robot.analog_data[0] = get_int(buffer[0], buffer[1]);// / 2048.0;
    robot.analog_data[1] = get_int(buffer[2], buffer[3]);/// 2048.0;
    robot.analog_data[2] = get_int(buffer[4], buffer[5]);/// 2048.0;
    robot.analog_data[3] = get_int(buffer[6], buffer[7]);/// 2048.0;
             
    /*
        if (telemetry_data.angle_z > 360)
        telemetry_data.angle_z -= 360;
        
        if (telemetry_data.angle_z < 0)
        telemetry_data.angle_z += 360;
 

    telemetry_data.vel_x  =  get_int(buffer[8], buffer[9]);/// 2048.0;
    telemetry_data.vel_y  =  get_int(buffer[10], buffer[11]);/// 2048.0;
      
        
    telemetry_data.update_rate = buffer[12] + buffer[13] * 256;
           */
           
    robot.serial_updates++;     
           
           
           
// Send the telmetry dfata to the client



    if (network_connected())
        robot_send_telemetry();
       
           
           
      
/*
    alert(MODULE_NAME "(%d) Analog:  %4.0f %4.0f %4.0f %4.0f\n", len,
                            robot.analog_data[0],   robot.analog_data[1],
                            robot.analog_data[2],   robot.analog_data[3]);                    

*/   
}
/* End of set_telemetry (buffer) */


/* Set new control data for Node IO board */
void robot_set_control(unsigned char * buffer, unsigned int len)
{
    robot.c_data[0] = buffer[0];
    robot.c_data[1] = buffer[1];
    robot.c_data[2] = buffer[2];
    robot.c_data[3] = buffer[3];
        
    robot.client_updates++; // we have new data, tell the failsafe
       
}
/* End of robot_set_controller */


/* Send motor and servo data to the serial port */
void robot_write_control( void )
{
    unsigned char buff[10];


//printf("Robot Write control %x %x %x %x\n", robot.c_data[0],
// robot.c_data[1], robot.c_data[2], robot.c_data[3]);

  
    buff[0] = robot.c_data[0];
    buff[1] = robot.c_data[1];
    buff[2] = robot.c_data[2];
    buff[3] = robot.c_data[3];
    
    serial_write_packet(COMM_NI_CTRL, buff, COMM_NI_CTRL_LEN);  
}
/* End of send_control_packet */


/* Reset the motor controller to the default positions */
void robot_reset( void )
{
    alert(MODULE_NAME "Reset\n");

    robot.c_data[0] = 0x80;
    robot.c_data[1] = 0x80;
    robot.c_data[2] = 0x80;
    robot.c_data[3] = 0x80;

    robot.client_updates = 0;
    robot.serial_updates = 0;
}
/* End of reset_controller */





/* Controller thread */
void *robot_thread( void *threadid )
{
    alert(MODULE_NAME "Thread\n");

    while(!program_done) 
    {
        usleep(TIME_SECOND / UPDATE_RATE);
          
        if (!serial_flushing())
        {
            robot_write_control();
                       
            unsigned char buff[COMM_IN_TEL_LEN + 1]; // Leave roomm for packet type
                       
            unsigned int ret;

            ret = serial_read_packet(buff, COMM_IN_TEL_LEN + 1);

            if (ret == 0)
            {
                if (buff[0] == COMM_IN_TEL)
                    robot_parse_telemetry(buff+1, COMM_IN_TEL_LEN); // Strip packet type
                else
                    alert(MODULE_NAME "robot_thread invalid packet type: %d\n", buff[0]);
            }
            
//            usleep(250000);            
            
            
        }
            
    }

    pthread_exit(NULL); // exit thread
}
/* End of controller_thread */


/* Start Controller */
void robot_start( void )
{
    if (ROBOT_ENABLE == 0) return;

    alert(MODULE_NAME "Start\n");

    serial_start();
    
    network_start();
    
    failsafe_start();
    
    robot_reset();

    pthread_create(&robot.robot_t, NULL, robot_thread, NULL);
}
/* End of start_controller */


/* Stop controller */
void robot_stop( void )
{
    if (ROBOT_ENABLE == 0) return;
    
    alert(MODULE_NAME "Stop\n");

    serial_stop();
}
/* End of controller_stop */






