/*
    NullBot Robot Platform - Server - 2005-2011 Jason Hunt
    
    GNU GENERAL PUBLIC LICENSE Version 2, June 1991
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: nulluser@gmail.com
    
    File: serial.h
*/

#ifndef SERIAL_H
#define SERIAL_H

#include <pthread.h>
#include <termios.h> 

#define I2C_ADDR 0x41
#define I2C_BYTE_PERIOD     1000        // interbyte delay for i2c

#define HEADER_SIZE 2
#define FOOTER_SIZE 1

#define DEVICE_CONT         1

#define DEVICE_CAM1         8
#define DEVICE_CAM2         9
#define DEVICE_CAM3         10
#define DEVICE_CAM4         11

#define DEVICE_DEBUG        15


#define COMM_PACKET_WAIT    0 
#define COMM_PACKET_DATA    1
#define COMM_PACKET_FLUSH   2

#define MAX_RX_PACKET       20



#define COMM_RX_MAX         512


#define BAUDRATE B57600


//#define FLUSH_PERIOD 13
#define FLUSH_PERIOD 8

#define SERIAL_MIN_PACKET  4




typedef struct
{
    pthread_t read_t;
    pthread_t monitor_t;
    

    int serial_fd;
    
    unsigned int state;

    unsigned int b_index;
    unsigned int packet_size;
    unsigned char rx_buff[COMM_RX_MAX];
    unsigned int flush;

    unsigned char log;

    struct termios save_options;

} serial_type;


int serial_flushing( void );

void serial_start( void );
void serial_stop( void );


int serial_read_packet( unsigned char *packet, unsigned int len );


int serial_write_packet( unsigned char packet_type, 
                         unsigned char *send_buff, 
                         int send_bytes );


#endif
