/*
	NullBot Robot Platform - NodeIO - 2005-2011 Jason Hunt

	Contact: nulluser@gmail.com
    
	GNU GENERAL PUBLIC LICENSE Version 2, June 1991
	
	This program is free software; you can redistribute it and/or
    	modify it under the terms of the GNU General Public License
    	as published by the Free Software Foundation; either version 2
    	of the License, or (at your option) any later version.

    	This program is distributed in the hope that it will be useful,
    	but WITHOUT ANY WARRANTY; without even the implied warranty of
    	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    	GNU General Public License for more details.

    	You should have received a copy of the GNU General Public License
    	along with this program; if not, write to the Free Software
    	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

	File: comm.c
*/


#include "core.h"


#define RX_BUFF_SIZE	16

// Finite State Machine states for serial
#define MODE_WAIT	0
#define MODE_DATA	2


static unsigned char rx_buff[RX_BUFF_SIZE];
static unsigned char cur_rx = 0;

static unsigned char comm_mode = MODE_WAIT;		// Current FSM mode for serial
static unsigned char packet_size;			// Size of this packet
static unsigned char device_id = 0;			// CUrrent device id
static unsigned char data_pending = 0;			// Data remaining in packet


unsigned char update_timeout = 0;
unsigned char timeout_count = 0;



unsigned char packet_timeout = 0;		// true if we failed to get a complete packet
unsigned char packet_time = 0;			// 


unsigned char sig_process = 0;






/* Compute checksum of packet */
unsigned int get_checksum(unsigned char *buff, unsigned int len)
{
	unsigned char ck = 0;
	
	while(len)
	{
		ck += buff[--len];
	}

	return(ck);
}
/* End of get_checksum */









void write_packet(unsigned char *packet, unsigned int len)
{
	unsigned int i;

	for (i = 0; i < len; i++)
		serial_tx(0, packet[i]);
}












/* Process packet data */
void process_packet( unsigned char *packet, unsigned char len )
{
	// validate checksum
	if (get_checksum(packet, len - 1) != packet[len - 1])
	{
		serial_timeout();
		fault |= FAULT_CF;
		return;
	}
	
	unsigned char packet_type = rx_buff[1];

	// Process the packet

	if (packet_type == COMM_NI_CTRL) 
		controller_packet(rx_buff + 2);		// Control packet, strip packet type and len
	else
	{	// Unknown packet

		fault |= FAULT_IP;
		serial_timeout();
		return;
	}



//	comm_clear_packet_timeout();
	

}
/* End of process packet */



/* Process byte from host */
void process_host_rx ( unsigned char d )
{

	if (comm_mode == MODE_WAIT)			// Waiting for packet size
	{

		comm_clear_packet_timeout();

		rx_buff[0] = d;				// Save byte

		cur_rx = 1;				// Reset buffer index

		packet_size = d;			// First byte is total packet size

		data_pending = packet_size - 1;		// We have the size, set data remaining

		if (data_pending >= 2)			// Wait more for pending > 0
		{
			comm_mode = MODE_DATA;
		} else
		{
			fault |= FAULT_IL;

		}

	} else

	if (comm_mode == MODE_DATA)			// Waiting for data
	{
		data_pending--;				// We got another byte
	
		rx_buff[cur_rx++] = d;			// Save byte
	
		if (cur_rx > RX_BUFF_SIZE - 1)		// Make sure we don't go over
			cur_rx = RX_BUFF_SIZE - 1;
		
		if (data_pending == 0)			// Process packet if we are done
		{
			sig_process = 1;
			
			comm_mode = MODE_WAIT;	// Back to wait mode
		}
	
	} 
}
/* End of process_host_rx */



/* Reset genreal comm */
void comm_reset( void )
{
	comm_mode = MODE_WAIT;
	packet_size = 0;	
	device_id = 0;		
	data_pending = 0;	
	cur_rx = 0;
}



void comm_clear_timeout( void )
{
	update_timeout = 0;
	timeout_count = 0;
}


void comm_clear_packet_timeout( void )
{
	packet_timeout = 0;
	packet_time = 0;
}



void comm_task( void )
{
	// Ready to transmit telemetry
	if(sig_process)
	{
		process_packet(rx_buff, packet_size);
		sig_process = 0;
	}

}




/* Update serial system */
void comm_timer( void )
{
	static unsigned char status_count = 0;


	if (!update_timeout)
	{

		timeout_count++;


		if (timeout_count >= TIMEOUT_PERIOD)
		{
			fault |= FAULT_UT;

			update_timeout = 1;
			 serial_timeout();
		}
	}




	if (!packet_timeout && comm_mode == MODE_DATA)
	{
		led = 0;

		packet_time++;


		if (packet_time >= TIMEOUT_PERIOD)
		{
			fault |= FAULT_PT;

	fault_data = packet_size;


			packet_timeout = 1;
			serial_timeout();
		}
	}


	// Check for error conditions
	if (update_timeout || packet_timeout)
	{
		if (++status_count > 50 && blink_enable )
		{
			status_count = 0;
			led = ~led;
		//	U1TXREG = 'A';
		}
	}
	else
		led = 0;

}
/* End of serial_timer */





/* Reset genreal comm */
void comm_init( void )
{
	comm_reset();


	// Setup Timer
	timer_register(comm_timer);
	task_register(comm_task);
}


